#!/usr/local/bin/perl 

# test.pl version 0.01
# Program to takes parallel text and determines the alignments
# and their probability for each sentence
#
# Copyright (C) 2002-2003
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Bridget McInnes, University of Minnesota, Duluth
# bthomson@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

#we have to use commandline options, so use the necessary package
use Getopt::Long;

#for LINUX because we are possible using french characters
use bytes;

#first check if no commandline options have been provided... in which case
#print out the usage notes
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

#now get the options
GetOptions(  "version", "help", "source=s", "target=s", "translationfile=s");

#if help has been requested, print out help
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

#if version has been requested, show version
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

#open the translation file
open(TRNS, $opt_translationfile) || die "Translation file not found.\n";

#open the sourcevcb file or die
open(SRC, $opt_source) || die "Source file not found.\n";

#open the targetvcb file or die
open(TRG, $opt_target) || die "Target file not found.\n";

#set distortion
$distortion = 8;

#if alignment file has been defined set alignmentfile
if ( defined $opt_wordalignmentfile ) { 
    $wordalignment = ">" . $opt_wordalignmentfile;
}
else {
    $wordalignment = ">" . $type . "." . $distortion . ".wa";  
} 

#open the wordalignment file
open(WA, $wordalignment);

%tp = ();

#read in the translation probabilities
print "Loading Translation Probabilities...\n";
#probabilities loaded as source target prob
while(<TRNS>) {
    chomp;
    @array = split/\s+/, $_;
    $tp{$array[0]}{$array[1]} = $array[2];
}
print "Translation Probabilities Loaded\n";

while(<SRC>) {

    $sourceline = $_;
    $targetline = <TRG>;

    chomp $sourceline;
    chomp $targetline;

    $sourceline =~s/<s snum=([0-9]+)>\s+//;
    $scount = $1;
    $sourceline =~s/<\/s>//;

    $targetline =~s/<s snum=([0-9]+)>\s+//;
    $tcount = $1;
    $targetline =~s/<\/s>//;

    if($tcount != $scount) { print "error! scount = $scount, tcount = $tcount\n\n"; exit; }

    @source = split/\s+/,$sourceline;
    @target = split/\s+/,$targetline;

    @n = ();
    for $i(0..$#target) { $n[$i] = 0; }

    for $i(0..$#source) {
	$maxprob = 0;
	$start = ($i - $distortion < 0) ? 0 : $i - $distortion;
	$end = ($i + $distortion > $#target) ? $#target : $i + $distortion;
	$tpos = $i+1;
	$spos = $i+1;
	for ($j = $end; $j >= $start; $j--) { 
	    if($maxprob < $tp{$source[$i]}{$target[$j]}) {
		$maxprob = $tp{$source[$i]}{$target[$j]};
		$tpos = $j+1;
	    }
	    if($maxprob == $tp{$source[$i]}{$target[$j]} && $n[$j+1] == 0) {
		$maxprob = $tp{$source[$i]}{$target[$j]};
		$tpos = $j+1;
	    }
	}
	$n[$tpos] = 1;
	print WA "$scount $spos $tpos\n";
    }
}

sub showVersion
{
    print STDERR "Copyright (C) 2003, Ted Pedersen & Bridget McInnes\n";
    print STDERR "test.pl  -  Version 0.01\n";
    print STDERR "30 February 2003\n";
}

#function to output "ask for help" message when the user's goofed up!
sub askHelp
{
    print STDERR "Type test.pl --help for help.\n";
}

#function to output the usage message when the user's goofed up
sub minimalUsageNotes
{
    print STDERR "Usage: test.pl [OPTIONS] --target FILE --source FILE --translationfile FILE\n";
    askHelp();
}

#function to output help messages for this program
sub showHelp
{
    print "Usage:  test.pl [OPTIONS] --target FILE --source FILE --translationfile FILE\n\n";
    print "OPTIONS:\n\n";

    print "  --version                   Prints the version number.\n\n";

    print "  --help                      Prints this help message.\n\n";

    print "REQUIRED:\n\n";

    print "  --source           FILE      Name of test source file.\n\n";

    print "  --target           FILE      Name of test target file.\n\n";

    print "  --translationfile  FILE      File containing the translation\n";
    print "                               table from model2.pl.\n\n";
}

